app1 <- function(tfs, tgs, a_tf, de_genes, M ){
    
#INPUTS:
  # tfs: list of TFs in ORTI, one at a line (can be either all TFs, or only with rank 1, or any subset of intrest)
  # tgs: list of TGs for each TF in tfs, i.e., row i contains the list of TGs of TF_i in the ith row of tfs. This may include either all TGs or only those with rank 1
  # de_genes: list of differentially expressed genes in the context of intrest, one gene at each line.
  # M: total number of TF-TG intreactions in ORTI: M = 10,370 if only Rank 1 pairs are considered (recommaded). M = 72,738 if all ranks are considered.
  # a_tf: tf of intrest

#OUTPUT:
  # pvals, the probabilities of TFs' overrepresentation in the context of intrest (unadjusted), i.e., line i gives the p-value of TF_i in tfs 

tf_index = match(a_tf, tfs[,1])
if(!is.na(tf_index)){
  tgs_i = t(tgs[tf_index,])   
  tgs_i = unique(tgs_i)  #tgs_i[tgs_i != ''] # remove empty cells
  N = length(intersect(as.matrix(unique(unlist(tgs))),as.matrix(de_genes)));
  k = length(tgs_i)-1
  x = length(intersect(tgs_i[,1],as.matrix(de_genes)))
  
  # p  = hygecdf(x,M,K,N) : p is the probability of drawing up to x of a possible 
  # K items in N drawings without replacement from a group of M objects.
  # M: population size (total number of TF-TG interactions)
  # K: total number of TGs annotated as being regulated by TF_i in  ORTI
  # (i.e.,  all TGs in  line i of tgs))
  # N: number of de_genes annotated as the TFs' targets in ORTI
  
  pval = dhyper(x, k, M-k, N,log = FALSE) + phyper(x, k, M-k, N, lower.tail = FALSE, log.p = FALSE);
}
else
  pval = NA

return(pval)
}
